@@ -1,21 +1,21 @@
 #####################################
 ## Robust SEM with missing data    ##
-## Ke-Hai Yuan and Zhiyong Zhang   ##
+## Zhiyong Zhang and Ke-Hai Yuan   ##
 ## Created: Dec 25, 2010           ##
 #####################################
 
 ## Version 2011/09/28
 ## Version 2011/12/11
+## Version 2012/06/07
+## Version 2012/07/21
 
 ##############################################
 ## Function to find missing data patterns   ##
 ## Function rsem.pattern                    ##
 ##############################################
-rsem.pattern<-function(x,n,p){
+rsem.pattern<-function(x,print=TRUE){
 ## This function generate missing data patterns
 ## INPUT
-## n: sample size
-## p: number of parameters
 ## x: data set
 ## OUTPUT
 ## misinfo
@@ -24,8 +24,11 @@ rsem.pattern<-function(x,n,p){
 ##   [3:(p+2), ] contains a permutation of {1,2,3,...,p} 
 ##                      the first part corresponding to observed variables
 ##                      the remaining corresponding to missing variables
-	if (missing(n)) n<-dim(x)[1]
-	if (missing(p)) p<-dim(x)[2]
+	if (missing(x)) stop("A data set has to be provided!")
+  if (!is.matrix(x)) x<-as.matrix(x)
+  
+  n<-dim(x)[1]
+	p<-dim(x)[2]
 	misorder<-rep(0,n)
 	for (i in 1:n){
 		misorderj<-0
@@ -86,11 +89,32 @@ rsem.pattern<-function(x,n,p){
 	}
 	patnobs<-c(patnobs, ncount)
 	patnobs<-patnobs[2:(totpat+1)]
-	if (is.vector(misinfo_0)) {misinfo<-c(patnobs, misinfo_0)}else{misinfo<-cbind(patnobs, misinfo_0)}
-	if (!is.matrix(misinfo)){misinfo<-matrix(misinfo, nrow=1)}
-	colnames(misinfo)<-NULL
-	rownames(misinfo)<-NULL
-	list(misinfo=misinfo, x=x)
+	if (is.vector(misinfo_0)){
+    misinfo<-c(patnobs, misinfo_0)
+  }else{
+    misinfo<-cbind(patnobs, misinfo_0)
+  }
+	if (!is.matrix(misinfo)){
+    misinfo<-matrix(misinfo, nrow=1)
+  }
+	
+	## Different presentation of missing data patterns
+  nr<-nrow(misinfo)
+  mispat<-matrix(1, nrow=nr, ncol=p)
+  
+  for (i in 1:nr){
+    if (misinfo[i,2]<p){
+      ind<-misinfo[i, (misinfo[i,2]+3):(p+2)]
+      mispat[i, ind]<-0
+    }
+  }
+  mispat<-cbind(misinfo[,1:2, drop=FALSE], mispat)
+  rownames(mispat)<-paste('Pattern ', 1:nr, sep="")
+  colnames(mispat)<-c('n','nvar',colnames(x))
+  
+  if (print) print(mispat)
+  
+	invisible(list(misinfo=misinfo, mispat=mispat, x=x))
 }
 
 ##############################################
@@ -107,11 +131,11 @@ rsem.ssq<-function(x){
 ## Function rsem.emmusig                    ##
 ##############################################
 
-rsem.emmusig<-function(x, misinfo, varphi=.1, max.it=1000, st='i'){
-## x: data set
-## misinfo: missing data pattern
-## varphi: 
-	
+rsem.emmusig<-function(xpattern, varphi=.1, max.it=1000, st='i'){
+  if (is.null(xpattern$mispat)) stop("The output from the function rsem.pattern is required")
+	x<-xpattern$x
+  misinfo<-xpattern$misinfo
+  
 	ep <- 1e-6  ## precision
 	n<-dim(x)[1]
 	p<-dim(x)[2]
@@ -263,6 +287,9 @@ rsem.emmusig<-function(x, misinfo, varphi=.1, max.it=1000, st='i'){
 		sig0<-sig1;
 		n_it<-n_it+1;
 	} ## end while
+  if (n_it>=max.it) warning("The maximum number of iteration was exceeded. Please increase max.it in the input.")
+  rownames(sig1)<-colnames(sig1)
+  names(mu1)<-colnames(sig1)
 	list(mu=mu1, sigma=sig1, max.it=n_it)
 }
 
@@ -395,7 +422,14 @@ rsem.switch<-function(p){
 ## covariance of \hat\Omega_{\hat\beta};    ##
 ## Function rsem.Ascov                      ##
 ##############################################
-rsem.Ascov<-function(x, mu0, sig0, misinfo, varphi=.1){
+rsem.Ascov<-function(xpattern, musig, varphi=.1){
+  if (is.null(xpattern$mispat)) stop("The output from the function rsem.pattern is required")
+  x<-xpattern$x
+  misinfo<-xpattern$misinfo
+  
+  mu0<-musig$mu
+  sig0<-musig$sig
+  
 	n<-dim(x)[1]; p<-dim(x)[2];
 	ps<-p*(p+1)/2; pps<-p+ps;
 	dup<-rsem.DP(p) ##duplication matrix
@@ -593,13 +627,29 @@ rsem.Ascov<-function(x, mu0, sig0, misinfo, varphi=.1){
 			snj<-snj+nj
 		} ##end for 2:npat
 	}
-	
+	rsem.gname<-function(name){
+    temp.name<-NULL
+    k<-length(name)
+    for (i in 1:k){
+      for (j in i:k){
+        temp.name<-c(temp.name, paste(name[i], ".", name[j], sep=""))
+      }
+    }
+    temp.name
+	}
 ## Constructing B_\bata and A_\beta matrices
 	Bbeta<-rbind( cbind(B11, B12), cbind(t(B12), B22) )
 	Abeta<-rbind( cbind(ddl11, ddl12), cbind(ddl21, ddl22) )
 	Abin<-solve(Abeta)
 	Omega<-n*Abin%*%Bbeta%*%t(Abin)
 	Gamma<-Omega
+  
+  xnames<-colnames(x)
+  if (is.null(xnames)) xnames<-paste('V', 1:p)
+  
+  mnames<-rsem.gname(xnames)
+  colnames(Abeta)<-colnames(Bbeta)<-colnames(Gamma)<-rownames(Abeta)<-rownames(Bbeta)<-rownames(Gamma)<-c(xnames, mnames)
+  
 	list(Abeta=Abeta, Bbeta=Bbeta, Gamma=Gamma)
 }
 
@@ -627,21 +677,21 @@ rsem<-function(dset, select, EQSmodel, moment=TRUE, varphi=.1, st='i', max.it=10
 	pvs<-p_v+p_v*(p_v+1)/2
 	
 ## missing data patterns
-	miss_pattern<-rsem.pattern(dset,n,p)
+	miss_pattern<-rsem.pattern(dset,print=FALSE)
 	x<-miss_pattern$x                             ## data after ordering
 	misinfo<-miss_pattern$misinfo                 ## missing data pattern
 	
 	totpat<-dim(misinfo)[1]                       ## total number of patterns
 	cat("There are", totpat, "missing data patterns. They are \n")
-	print(misinfo)
+	print(miss_pattern$mispat)
 	cat("\n")
 ## run EM
-	em_results<-rsem.emmusig(x,misinfo,varphi, max.it, st)
+	em_results<-rsem.emmusig(miss_pattern,varphi, max.it, st)
 	if (em_results$max.it >= max.it){ warning("\nMaximum iteration for EM is exceeded and the results may not be trusted. Change max.it to a greater number.\n") }
 	hmu1<-em_results$mu                           ## means for all variables including auxiliary variables
 	hsigma1<-em_results$sigma                     ## covariance matrix for all variables
 ## Calculate the sandwitch covariance matrix
-	ascov_results<-rsem.Ascov(x, hmu1, hsigma1, misinfo, varphi)
+	ascov_results<-rsem.Ascov(miss_pattern, em_results, varphi)
 	Abeta<-ascov_results$Abeta
 	Bbeta<-ascov_results$Bbeta
 	hupsilon<-ascov_results$Gamma
